#!/usr/bin/perl -w
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

#####
# inbound.cgi
#    session id setting and redirecting script
#    RF 981208 - based on sid.cgi, version 1-981208
#
#####

####
#
# Version 0.7.12 IB 991116
#    corrected the number of arguments in a database call
# Version 0.7.11 IB 991113
#    removed cgi-lib.pm because it is not needed
# Version 0.7.10 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.9 IB 991111
#    fixed list of libraries
# Version 0.7.8 IB 991110
#    correction of a database call
# Version 0.7.7 IB 991109
#    removing of HACKS, and correcting of relpath
# Version 0.7.6 IB 991025
#    the UID field has been dropped from table clicks
# Version 0.7.5 RF 991022
#    fixed path failure to static pages
# Version 0.7.4 RF 991022
#    fixed redirection failure to getpage.cgi
# Version 0.7.3 RF 991022
#    added patch to older store.pm - assume database name is
#    also the name of the directory below ./cgi-bin in the 
#    tree, and provide that as an argument to setstore()
#    Also derefed the $store hashref to satisfy older libraries
# Version 0.7.2 RF 991022
#    near total rewrite
#    added relative path detection for requires statements
#    trimmed diagnostic printing
#    added logic to spot non-cookie taking customers
# Version 0.7.1 GF 990830
#    trying to revitalize this script
# Version 4 - 981218 
#    efficiency improvements in prep for eWallet hit
# Version 2 - 981213
#    added logging of inbounds...
# Version 1 - 981208
#
#####

$sid=$ENV{'HTTP_COOKIE'};
$sid=( split /sid=/, $sid )[1] if ( $sid );
$sid=( split /&/, $sid )[0] if ( $sid );

## determine path from store root and store db name...
#
#  note - this script runs at server root or in cgi-bin/om,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$storeshortname="";
## short circuit if $s does not contain cgi-bin
if ( $s !~ /cgi-bin/ ) {
   $storeshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "cgi-bin" ) {
      $storeshortname = $temp[$#temp];
      @temp=();
   } # note - cgi-bin is off the site root, so the next up is site root

} # done constructing relative path to store root

use DBI;
require "$relpath/conf/store.pm";
require "$relpath/lib/state.pm";

## establish store-specific parameters
#
$site=setstore();

$now=time();

## open database connection
#
$q="DBI:$site->{'datadriver'}:$site->{'table'}:$site->{'table_serv'}:$site->{'table_port'}";
$dbh=DBI->connect($q, $site->{'store_dbiuser'}, $site->{'store_dbipwd'} );

## now either have a sid or not...
#
if ( $sid ) {         # session id cookie present
   # have a cookie, just send them to their requested page with simple logging
   #
   $target = ( split /page=/,$ENV{'REQUEST_URI'},1 )[1];
   $target =~ s/&cookietest=on// ;

   $target="/html/home.shtml" if ( ! $target );            # default to homepage 

   ## log the click to the table of clicks
   #
   $q="INSERT INTO clicks VALUES ";
   $q.=" ($sid,$now,'$ENV{'REQUEST_URI'}','$ENV{'REFERER'}','$site->{'clusterid'}')";
   $dbh->do( $q );

   print "Status: 302/Moved\n";
   print "Location: $target\n";    
   print "\n\n";

} else {              # session id cookie absent
   # do not have a cookie, check for second pass cookie test
   #
   # TBD - for getpage.cgi:
   #    test for already wrapperized customer, strip sid if coming in from
   #    outside the site (someone bookmarked a wrapperized session)
   #
   # check for explicit sid passed as a GET method
   #

   $ENV{'REQUEST_URI'}="" if ( ! $ENV{'REQUEST_URI'} ); 

   if ( $ENV{'REQUEST_URI'}=~/sid=/ ) {                 # sid embeded in URI
      $sid=$ENV{'REQUEST_URI'};
      $sid=( split /sid=/, $sid )[1];
      $sid=( split /&/, $sid )[0] if ( $sid );
   } # done looking for embeded sid

   if ( $ENV{'REQUEST_URI'} =~ /cookietest=on/ )  {       # we just tried to give them a cookie, and failed
      $target = ( split /page=/,$ENV{'REQUEST_URI'},1 )[1];
      $target="home.shtml" if ( ! $target );            # default to homepage 
      
      ## redirect them to the getpage.cgi widget for wrapperization
      #
      print "Status: 302/Moved\n";
      print "Location: /cgi-bin/om/getpage.cgi?&page=$target\n";
      print "\n\n";

   } else {                                             # give them a cookie and redirect them back here

      $target = $ENV{'REQUEST_URI'};
      $target="" if ( $target eq "/" );

      if ( ! $sid ) {                                   # set a cookie
         $sid=setsessionid();                           # assign a cookie
      } else {                                          # set a cookie with sid salvaged from URI
         print "Set-Cookie: sid=$sid; path=/; domain=$ENV{'SERVER_NAME'}\n";
      } # done with selection of how to set the cookie 
      $target.="&cookietest=on";                        # add the cookietest flag
      print "Status: 302/Moved\n";
      print "Location: /cgi-bin/om/inbound.cgi?&page=$target\n";
      print "\n\n";

      $q="INSERT INTO clicks VALUES ";
      $q.=" ($sid,$now,'$ENV{'REQUEST_URI'}','$ENV{'REFERER'}','$site->{'clusterid'}')";
      $dbh->do( $q );

   } # done with cookie acceptance testing for cookieless sessions 

} # done with examining session id cookie present or absent

## cleanup statement handles and close connection
#
$sth->finish if ( $sth );
$dbh->disconnect if ( $dbh );

1;

