#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# generate_temps.pl
#
# Program to generate temporary data files
#    to feed into various general pulldowns
# 
# Version 0.7.2 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    calculation of a relative path
#    using of new global variables
# Version 19 RF 990908
#    relative path symlinks - removed absolute symlink paths
#    changed libraries to relative paths
#    added test for handler-less sku
#
# Version 12 SH 990707
#    moved to system 6 data structure and queries
#    if encoding does not have relevant field, sku.sel is created
# Version 11 GF 990310
#    fixed problem regarding nobody running the script
# Version 10 RF 981219
#    removed refs to $datahost
#    removed all calls to machine.pm
# Version 9 RF 981127
#    fixed generation of pulldowns
# Version 8 RF 981124
#    added fieldname binding detection, generates fieldname.sel
#    files as appropriate in addition to XEC sku filenames
# Version 7 RF 981123
#    takes a storecode as an argument
# Version 6 RF 981122
#    multi-store capacity and bits/sel useage
# Version 5 RF 981111
#    converted to DBI interface
# Version 4 RF 981103 
#    added sku.sel generation for encodings
# Version 3 RF 981103 lost
# Version 2 RF 981013
#    added string conversion from mysql to text
# Version 1 RF 981012
#

## determine path from store root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$storeshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $storeshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $storeshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

use DBI;

require "$relpath/lib/formtools.pm";
require "$relpath/conf/store.pm";

## establish store-specific parameters
#
$site=setstore();

open(ERLOG,">/tmp/generate_temps.pl");

## establish connection in
$s = "DBI:$site->{'datadriver'}:$site->{'table'}:$site->{'table_serv'}:$site->{'table_port'}";
print ERLOG "connecting to $s \n";
$dbh = DBI->connect($s, $site->{'store_dbiuser'}, $site->{'store_dbipwd'});
print ERLOG "connected to $s \n";

$datapath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/site/sel";

chdir( $datapath );          # move into the datapath directory
print ERLOG "Changing to directory $datapath \n";

#
## generate brands pulldown data file
#
open(FH, ">$datapath/BRAND.sel") || die "Unable to open $datapath/BRAND.sel";
$q="SELECT SKU,AUTHOR,TITLE from things where SKU LIKE 'xbr%'  ";
$q.=" and ( STATUS<>'px' ) and ( STATUS<>'pd' ) ";
$q.=" ORDER BY TITLE ";
$sth=$dbh->prepare($q);
$sth->execute;
$num=$sth->rows;
for ($x=0; $x<$num; $x++ ) {
   ($k,$f,$v) = $sth->fetchrow_array;
   $v=string_exmysql($v);
   $f.=$storecode;
   print FH "$k\|$f $v \n";
}
close(FH);

#
## generate encoding-specific pulldown .sel files
#
$q="SELECT OBJID,BODY from extraprop where OBJID='$ARGV[1]' and FIELDNAME='DESCRIPT'";
$sth=$dbh->prepare($q);
$sth->execute;
$num=$sth->rows;
open (ERRL,">/tmp/gs");
print ERRL "first q: $num\n";
for ($x=0; $x<$num; $x++ ) {
   ($k,$v) = $sth->fetchrow_array;
   $target="$k".".sel";
   open(FH, ">$datapath/$target") || die "Unable to open $datapath/$target";
   print FH " \|- Not Specified\n";
   $v =~ s/\015/\n/sg ;
   $v =~ s/\012/\n/sg ;
   $v=string_exmysql($v);
   ## replace first colon with vertical slash
   #
   $v =~ s/(^\S+):/$1\|/mg ;
   $v =~ s/^\s+//mg ;


   ## HACK to get rid of blank lines in the string...
   @temp = split /\n/, $v;
   @temp2=();
   while (@temp) {
      $k = shift @temp;
      push @temp2,$k  if ( $k );
   }
   $v = join "\n", @temp2;
   print FH "$v";
   close(FH);
}

#
## copy the encoding-specific pulldown files as FIELDNAME.sel
#
$q="SELECT SKU, HANDLER from things where SKU='$ARGV[1]' ";
$sth = $dbh->prepare($q);
$sth->execute;
$num=$sth->rows;
print ERRL "second q: $num\n";
for ( $x=0; $x<$num; $x++ ) {
   ($k,$v) = $sth->fetchrow_array;
   if (!$v) { $v=$k };                # if no handler, use sku as handler
   $v=string_exmysql($v);            # graceful handling of underscore
   print ERLOG "$k covers $v \n";
  
   # these things should be lowercase if field names
   $target=$v.".sel";
   $source=$k.".sel"; 
   if ( -e "$datapath/$source" ) {  # source exists
      `/bin/ln -s --f $source $target` if ( $target ne $source );
   } # end of testing for sourcefile existence
} # done running through encoding skus


$sth->finish;
$dbh->disconnect if ( $dbh );

`chown nobody:nobody $datapath/*.sel`;

1;
