#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# xml_order_export.pl
#
# Program to create xml order files for transfer to warehouse
#    arguments: storecode, orderid
#
# Version 0.7.3 IB 991115
#    correction of database cleanup
# Version 0.7.2 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    calculation of a relative path
#    using of new global variables
# Version 5 RF 991012
#    added status of "xd" = xml dumped
#    changed to take HQUANT+SQUANT as quantity to ship
#    changed to ignore fulfillable or not
# Version 4 RF 990921
#    added objects for use in XML templates:
#       vsku_uc   - uppercase VSKU
#       lineid_uc - uppercase LINEID
#       oid_uc    - uppercase OID/ORDERID
# Version 3 RF 990921
#    readout of order creation date from timestamp
#    fixed retrieval of order line items
# Version 2 RF 990921
#    better logging of error conditions
#    fixed absence of store fsrelativeroot from template load path
# Version 1 RF 990921
#
#

## determine path from store root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$storeshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $storeshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $storeshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root


use DBI;
require "$relpath/conf/store.pm";
require "$relpath/lib/xml.pm";

## establish store-specific parameters
#
$site=setstore();
chdir "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/crons/";

@orders=@ARGV[1..$#ARGV];         # slice containing orders

open (ERLOG, ">/tmp/xml_order_export.pl");

## establish connection in
#
$s = "DBI:$site->{'datadriver'}:$site->{'table'}:$site->{'table_serv'}:$site->{'table_port'}";
$dbh = DBI->connect($s, $site->{'store_dbiuser'}, $site->{'store_dbipwd'});

$targetpath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/transit/export/orders";

## Load templates for output XML
#
%template=();

$template{'BOD36'}=xml_loadtemplate( "BOD36", $absroot ); 
$template{'ADDRESS'}=xml_loadtemplate( "ADDRESS", $absroot ); 
$template{'DATETIME'}=xml_loadtemplate( "DATETIME", $absroot ); 
$template{'PICKLINE'}=xml_loadtemplate( "PICKLINE", $absroot ); 


## report loading errors
#
if ( ERLOG ) {
   foreach $k ( keys %template ) {
      if ( $template{$k}->{'error'} ) {
         print ERLOG "ERROR LOADING TEMPLATE $k $template{$k}->{'error'} \n";
      } # done examining error messages
   } # done running through templates that need loaded
} # done with "if there is an error log..."



########### general flow
#
# validation of input args ( can be order list )
#
# for each:
#    test for existing output file
#    test for complete export capability
#    generate event for transit file generation or failure
#    update status of order to "sent to warehouse"
# 
#

foreach $order ( @ orders ) {

   $curord={};                        # current order hashref
   $blocks={};                        # blocks being assembled
   $order_is_shippable=1;             # default assume order is pickable

   ## pull order header information
   #
   $t=$order;
   $t =~ s/\-//;
   $q="SELECT \* from orders where OID LIKE '$t%' ";
   $sth = $dbh->prepare( $q );
   $sth->execute;
   $num=$sth->rows;
   print ERLOG "$num rows found matching $q \n";
   if ( $num >= 1 ) {      # one order found...
      $curord = $sth->fetchrow_hashref;
      $c = ( keys %{ $curord } );
      print ERLOG "$c fields loaded from one record! \n";
   } else {                # this is an error - wrong record count for order
      ## TBD - error handling 
      print ERLOG "In ERROR for $t (was $order)! \n";
   } # done pulling order header data
   $sth->finish;

   ## uppercased order pseudo-fields
   #
   $curord->{'oid_uc'}=uc( $curord->{'OID'} );

   ## construct timestamps
   #
   $h={};
   @time=localtime( $curord->{'CDATE'} );
   $time[5]+=1900;                   # add century offset
   $time[4]++;                       # month 0 is Jan, change so 1 is Jan
   @ntime=("second","minute","hour","day_om","month","year");
   foreach $i ( 0..5 ) {
      $h->{$ntime[$i]}=$time[$i];
   } # done pushing time elements up
   $h->{'subsecond'}="1000";
   $h->{'qualifier'}="CREATION";
   $blocks->{'datetime:CREATION'}=xml_permute( $template{'DATETIME'}->{'skel'}, $h );

   $h={};
   @time=localtime( time() );
   $time[5]+=1900;                   # add century offset
   $time[4]++;                       # month 0 is Jan, change so 1 is Jan
   @ntime=("second","minute","hour","day_om","month","year");
   foreach $i ( 0..5 ) {
      $h->{$ntime[$i]}=$time[$i];
   } # done pushing time elements up
   $h->{'subsecond'}="1000";
   $h->{'qualifier'}="DOCUMENT";
   $blocks->{'datetime:DOCUMENT'}=xml_permute( $template{'DATETIME'}->{'skel'}, $h );

   ## construct shipto and customer
   #
   $blocks->{'partner:ShipTo'}="";
   $blocks->{'partner:Customer'}="";
   $q="SELECT \* from addresses where ADDRID='$curord->{'SHIPTO'}'" ;
   $sth = $dbh->prepare( $q );
   $sth->execute;
   $num=$sth->rows;
   if ( $num > 0 ) {    # got at least one address, use one
      $h = $sth->fetchrow_hashref;
      $h->{'partnertype'}="ShipTo";
      $blocks->{'partner:ShipTo'}.=xml_permute( $template{'ADDRESS'}->{'skel'}, $h );
      $h->{'partnertype'}="Customer";
      $blocks->{'partner:Customer'}.=xml_permute( $template{'ADDRESS'}->{'skel'}, $h );
      print ERLOG "OK SHIPTO for $q (order $t $order)! \n";
   } else {             # need error trapping - bad address
      ## TBD - error trap and handling for zero address records
      print ERLOG "ERROR -- no SHIPTO for $q (order $t $order)! \n";
   } # done pulling shipping address data

   ## construct picklines
   #
   $blocks->{'picklines'}="";
   $q="SELECT \* from ocart where OID='$order' ";
   $sth = $dbh->prepare( $q );
   $sth->execute;
   $num=$sth->rows;
   print ERLOG "found $num line items \n" if ( ERLOG );
   foreach ( 1..$num ) {
      ## pull each line item, check for SQUANT>0 as an error
      # 
      $h = $sth->fetchrow_hashref;

      ## ignore fillability as of v5 until better inventory data available 
      #
      # commented $order_is_shippable=0 if ( $h->{'SQUANT'}>0 );   # FLAG as unfillable order

      ## construct VSKU pseudo-field and populate uppercased pseudofields
      #
      $h->{'VSKU'}=$h->{'SKU'}.$h->{'VCODE'};
      $h->{'vsku_uc'}=uc( $h->{'VSKU'} );
      $h->{'lineid_uc'}=uc( $h->{'LINEID'} );

      ## TBD - retrieve sku title and variant encoding expansion
      # 
      $h->{'TITLE'}="product name unavailable";
      $q2="SELECT TITLE from things where SKU='$h->{'SKU'}'";
      $stt=$dbh->prepare( $q2 );
      $stt->execute;
      if ( $stt->rows > 0 ) {
         $h->{'TITLE'}=$stt->fetchrow_hashref->{'TITLE'};
      } # done loading title, if possible


      ## construct XML pickline
      #
      $blocks->{'picklines'}.=xml_permute( $template{'PICKLINE'}->{'skel'}, $h );
   } # /done assembling rows

   ## last minute additions to curord
   #  
   $curord->{'revisionid'}="$time[0]";
   $curord->{'storeshortname'}="MyHome";

   $output=$template{'BOD36'}->{'skel'};

   $c = ( keys %{ $curord } );
   print ERLOG "$c fields sent to output merge! \n";

   $output=xml_permute( $output, $blocks );
   $output=xml_permute( $output, $curord );

   ## test to see if generating a free-standing order file or not
   #
   if ( $order_is_shippable ) {                # free standing orderfile
      open( FH, ">$targetpath/$t.xml" );
      print ERLOG "$targetpath/$t.xml creation \n"; 
      print FH "$output \n";
      close( FH );
      ## TBD - generate event and change order status flag
      #
      
      $q = "UPDATE orders SET STATUS='xd' where OID='$curord->{'OID'}' and STATUS!='xd' ";
      $dbh->do( $q ); 

   } else {                                    # unshippable just yet
      print ERLOG "########################### \n";
      print ERLOG "$output \n";
      ## TBD - generate export failure event
      #
   } # done with deciding if order is fulfillable or not

} # done examining each order requested
#

close( ERLOG );

1;
