#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
# make_log_digest.pl
# RF 990416
#
# script to take a logfile and prepare  digest of it:
#    hits
#    pageviews
#    session ids
#    pageviews by pagename
#     
# Version 6 RF 991015
#    converted to new directory structures
#       -> use of transit/export/log
#       -> use of transit/import/trigger
# Version 5 RF 990419
#    added cleanup of outboundlogs based on contents of /www/readback
# Version 4 RF 990419
#    added compression of logs and movement into /www/outboundlogs
# Version 3 RF 990419
#    fixed bug in pageview counter
# Version 2 RF 990416
#    excluded error logs from processing
#    corrected wording of hits/pageviews
#    TBD - 
#       zip primary logs when done
#       move processed files to an archive or upload point?
# Version 1 RF 990416
#
#
# BUGS and TO-DO
#   1. directory hardcoded
#   2. looks for logfiles ending in .YYMMDD_HHMM for processing
#          then renames it to host.filename, moves it to the
#          /usr/local/home/sysproc/movelogs/outbound directory
#          and puts the digest there as well
#   3. referer data and proper sessioning
#

$server_root="/www/stronghold";
$server_logs="$server_root/logs";
$pickuppoint="/www/outboundlogs";
$confirmdrop="/www/readback";
$server_log_archive="/www/logarchive";

## construct just the host portion of the fully qualified hostname
#
$hostname=`/bin/hostname`;
$hostname = ( split /\./, $hostname )[0];

## cleanup of transit directory
#
@files=();                  # empty list of logfiles to move
opendir LOGS, $confirmdrop;
@allfiles = readdir LOGS;
closedir LOGS;

foreach $f ( @allfiles ) {
   next if ( $f !~ /^ok\./ );     # if not an "ok" file ...
   `/bin/rm $confirmdrop/$f`;     # delete "ok" file
   $f =~ s/^ok\.// ;              # remove the leading "ok"
   `/bin/mv $pickuppoint/$f $server_log_archive/`; # to the archive...
} # done digging through the readbacks

## move into logs directory and move new files 
#
@files=();                  # empty list of logfiles to move
opendir LOGS, $server_logs;
@allfiles = readdir LOGS;
closedir LOGS;

while (@allfiles) {
   $f = pop @allfiles;
   next if ( $f !~ /^sh_/ );                   # all logs start with this
   next if ( $f !~ /\d\d\d\d\d\d_\d\d\d\d/ );  # skip non-timestamped logs
   next if ( $f !~ /transfers/ );              # avoid the error logs
   push @files, $f;
} # done looking through file list


## hack for testing
# @files= ( "sh_homepage_transfers.990316_0100" );

while (@files) {               # start moving files
   $fstart = pop @files;
   $fend = "$hostname.$fstart";
   $fdigest="$fend".".dig";

   ## initialize hashes
   #
   %refer_p=();             # referers
   %view_p=();              # views by pagename
   %search_t=();            # searches by searchterm
   ## scalars
   #
   $hits=0;
   $pgvw=0;

   open(FH, "<$server_logs/$fstart" );
   while (<FH>) {
      $hits++;            # increment the hits counter
      $l = $_ ;
      @pn =  ( $l=~/"GET.+?"/g ) ;
      @pn =  ( $l=~/"POST.+?"/g ) if ( ! @pn );
      # now get the requested page name 
      $pr = ( split /\s+/, $pn[0] )[1];

      ## what kind of request?
      #
      # $pr = "z_image_hits" if ( $pr =~ /(gif|jpg)$/ ) ;
      if ( $pr =~ /(gif|jpg)$/ ) {
         $pr = "z_image_hits" ;
      } else {
         $pgvw++;                    # it is a pageview!
      } # done determining if an image hit or pageview
      $st="";
      ( $pr,$st ) = ( split /\?/, $pr );
      if ( $st ) {
         ( $k,$st ) = split /keywords=/, $st;
         ( $st,$k ) = split /\&/, $st;
         $st="-" if ( ! $st );
         # print " $st \n";
         $search_t{$st}++;
      }
      $view_p{$pr}++;

   } # end of reading through lines of logfile
   close(FH);

   open(FO , ">$server_logs/$fdigest");
   print FO "$hostname \n";

   ## now dump statistics
   #
   print FO "Basic Server Stats:\n";
   print FO "   Hits:                $hits \n";
   print FO "   Page Views:          $pgvw \n";
   $pgvw = $view_p{'/'};
   print FO "   Root Homepage Views: $pgvw \n";
   $pgvw = $view_p{'/cgi-bin/register.cgi'} ;
   print FO "   Signup Thanks/Error: $pgvw \n";
   print FO "\n\n";

   print FO "Pageviews by pagename:\n";
   @temp = sort keys %view_p;
   foreach $k ( @temp ) {
      print FO "   $view_p{$k} views $k \n";
   } #

   print FO "Search Terms:\n";
   @temp = sort keys %search_t;
   foreach $k ( @temp ) {
      print FO "   $search_t{$k} searches: $k \n";
   } #

   ## cleanup
   #
   close(FO);

   ## move logfile
   next if ( -e "$server_logs/$fend" );
   `/bin/mv $server_logs/$fstart $server_logs/$fend`;  # rename host.
   `/bin/gzip $server_logs/$fend`;                     # compress log
   $fend.=".gz";
   `/bin/mv $server_logs/$fend $pickuppoint/$fend`;  # pickup
   `/bin/chown logmove:nobody $pickuppoint/$fend`;  # logmove owns it
   `/bin/mv $server_logs/$fdigest $pickuppoint/$fdigest`;  # pickup
   `/bin/chown logmove:nobody $pickuppoint/$fdigest`;  # logmove owns it

}                              # done moving files


1;



