#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

#####
# revert_carts.pl
#
# Program to revert overly-old shopping carts from checkout
#    RF 981213 
#
# Version 0.7.3 IB 991217
#    eliminated ERLOG
# Version 0.7.2 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991110
#    correcting calls to database
# Version 2 SH 990707
#    moved to system 6 data structure and queries
#    ccart now becomes tcart
#    inventory is inserted back
#
# Version 1 RF 981213

## determine path from store root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$storeshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $storeshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $storeshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

$started=(times)[0];

use DBI;
require "$relpath/lib/store.pm";
require "$relpath/lib/state.pm";

## establish store-specific parameters
#
$site=setstore();

$store_tcart_ttl=10;          # number of sconds before ccart->scart

## establish connection in
#
$s = "DBI:$site->{'datadriver'}:$site->{'table'}:$site->{'table_serv'}:$site->{'table_port'}";
$dbs = DBI->connect($s, $site->{'store_dbiuser'}, $site->{'store_dbipwd'} );

$now=time();                      # now
$then=$now-$store_tcart_ttl;      # carts older than this revert

########### general flow
##  go through tcart, find all sids
##  for each sid, get mdate
##  for each sid where mdate older than $store_tcart_ttl, revert
##
##########

%sids=();
$q='SELECT SID,MDATE from tcart';
$sts = $dbs->prepare( $q );
$sts->execute;
$num=$sts->rows;
foreach ( 1..$num ) {
   ( $s,$m ) = $sts->fetchrow;

   if ( exists($sids{$s}) ) {
      $sids{$s}=$m if ( $m > $sids{$s} );

   } else {
      $sids{$s}=$m;

   } # done with exists or not

} # done running through ccart

## now go through the sids and revert old carts
#
foreach ( keys %sids ) {
  $q="SELECT MDATE from sessions where SID=$_";
  $sts = $dbs->prepare( $q );
  $sts->execute;

  if ( $sts->rows==1 ) {
     ( $m ) = $sts->fetchrow;
      $sids{$_}=$m if ( $m > $sids{$_} );

  } # done looking at the mdate     

  if ( $sids{$_}<$then ) {         # session inactive too long, revert
     &tcart_to_inventory( $_ );

  } # done with reversion cycle for this sid

} # done running through sids with ccart items and reverting


1;
