#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
# roll_logs.pl
# RF 990414
# script to rotate the log files and then hup the server
#
# Version 2 RF 991015
#    fixed month number for human readability
# Version 1 RF 990414
#
# GENERAL OVERVIEW:
#    this program creates a timestamp to tag onto filenames,
# then goes through the log directories for stronghold and 
# locates all non-timestamped log files. These files are moved
# to timestamped filenames and the server given a reload.
# Files are then chowned to nobody and moved to the upload
# point for movement to the central processing point.
# Finally, the logfile analysis script is triggered.
#
# KNOWN BUGS and THINGS TO FIX:
#    - paths are hardcoded for the moment
#    - no detection of filename collisions
#    - no monitoring of system load
#

## initialize timestamp, hostname, and path variables
#
$now=time();                # seconds since Jan 0, 1970...
@now=localtime($now);       # nicer date bits
$hostname=`/bin/hostname`;

$server_root="/www/stronghold";
$server_logs="$server_root/logs";
$server_hup="$server_root/bin/reload-server";

## construct timestamp for inclusion in logfile filenames
#
$now[4]++;             # add one to default month numbers, i.e. Jan 0->1
foreach $i ( 1..5 ) {
   $now[$i] = "00$now[$i]";
   $now[$i] = substr( $now[$i], length($now[$i])-2, 2 );
} # done padding with leading zeroes
$timestamp = "$now[5]$now[4]$now[3]"."_"."$now[2]$now[1]";

## move into logs directory and move files 
#
@files=();                  # empty list of logfiles to move
opendir LOGS, $server_logs;
@allfiles = readdir LOGS;
closedir LOGS;

while (@allfiles) {
   $f = pop @allfiles;
   next if ( $f !~ /^sh_/ );                   # all logs start with this
   next if ( $f =~ /\d\d\d\d\d\d_\d\d\d\d/ );  # skip timestamped logs
   push @files, $f;
} # done looking through file list

while (@files) {               # start moving files
   $f = pop @files;
   $n = "$f.$timestamp";
   `/bin/mv $server_logs/$f $server_logs/$n`;
}                              # done moving files

`$server_hup`;                 # restart server

1;



