#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# run_stats.pl
#
# Create daily summary report on core statistics
#    arguments: storecode [, YYYYMMDD]
#
# Version 0.7.2 IB 991112
#    corrected the determining of a relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    removing of HACKS, and correcting of relpath
# Version 2 RF 991108
#    rearangement of hash ordering and prelim dump
# Version 1 RF 991104
#    creation
#

## determine path from store root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$storeshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $storeshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $storeshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

use DBI;
require "$relpath/conf/store.pm";

## establish store-specific parameters
#
$site=setstore();

## establish connection in
#
$s = "DBI:$site->{'datadriver'}:$site->{'table'}:$site->{'table_serv'}:$site->{'table_port'}";
$dbh = DBI->connect($s, $site->{'store_dbiuser'}, $site->{'store_dbipwd'});

$targetpath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/html/stats";

## Load templates for output HTML
#
%template=();

########### general flow
#
# validation of input args ( can be order list )
#    fill in defaults for store/date
#
# select all sessions for date
#    determine sessions per referer
#    determine sessions per adbuy
#    determine sessions per artwork
#    determine sessions per affiliate
#    determine sessions per browser
# 
# select all pageviews
#    determine pagedepth per session
#    determine total number of page views
#
# select all orders
#    determine total revenues
#    determine total line items
#    determine total items
#    determine total shipping
#    determine total tax
#
# Derivative Values
#    average pagedepth = pageviews/sessions
#    conversion rate = sessions/orders
#    

## initialize tallying hash
#
%tot=();

## construct timestamps
#
$time_n=time();
if ( $#ARGV >= 1 ) {
   # there was a second argument, see if it is numeric...
   # TBD - what formats are acceptable?
   #	$test_str = $ARGV[1];
#	$year  = $test_str =~ m/.{4}$/  ;
	
#	if (($year < 1970) or ($year > 2100)) {
#			print "ERROR: invalid year format yyyymmdd\n";
#			exit 0;
#		}
### for now the number is the int days back	
	$days = $ARGV[1];
}

## construct timestamp of midnight last
#
for $d (0..$days){

if ($time_i) {$time_n = $time_i - (24 * 60 * 60);}
@ltime=localtime($time_n);
$time_i=$time_n; 
if ( ($ltime[0]+$ltime[1]+$ltime[2])!= 0 ) {
   $time_i=$time_n - $ltime[0] - ( $ltime[1]*60 ) - ( $ltime[2]*3600 )  ;
} # done testing for midnight
## construct timestamp of midnight next
$time_f=$time_i + ( 24 * 60 * 60 );       # midnights are always 24hours apart
 
## sessions
#
# initialize session hash
%session=();
# initialize counter hashes
%ses_ref=();               # by referer
%ses_adb=();               # by adbuy
%ses_art=();               # by banner artwork
%ses_aff=();               # by affiliate code
%ses_bro=();               # by browser
%ses_newu=();		   # by SID w/o UserID
%ses_oldu=();		   # by SID w UserID
$ses_newu{'TOTAL'} =0;
$ses_oldu{'TOTAL'} =0;

$q="SELECT \* from sessions where CDATE>=$time_i and CDATE<$time_f ";
$sth = $dbh->prepare( $q );
$sth->execute;
$num=$sth->rows;
$tot{'sessions_raw'}=$num;       # total sids burned through
$tot{'sessions_ok'}=0;           # ignoring sessions from within site
foreach ( 1..$num ) {
   $h = $sth->fetchrow_hashref;

   ## filter out internally referred sessions
   #
   next if ( $h->{'REFERER'} =~ /($site->{'storeurl'}|$site->{'storesslurl'})/ );
   $tot{'sessions_ok'}++;
 
   ## store session minimum data
   #
   $session{$h->{'SID'}}=$h;         # HACK - this will explode for large sessions...

   ## test for existence of counters
   #
   if ( $h->{'SID'}) {
      if ( $h->{'UID'}) {
       if ( ! $ses_oldu{$h->{'UID'}} ){
	$ses_oldu{$h->{'UID'}} = 1;
	$ses_oldu{'TOTAL'}++;
	}
      }else{
	
       if ( ! $ses_newu{$h->{'SID'}} ){
	$ses_newu{$h->{'SID'}} = 1;
	$ses_newu{'TOTAL'}++;
      }	
   }
}

   if ( $h->{'REFERER'} ) {          # by referer
      $ses_ref{$h->{'REFERER'}}=0 if ( ! $ses_ref{$h->{'REFERER'}} );
      $ses_ref{$h->{'REFERER'}}++ if ( $ses_ref{$h->{'REFERER'}} );
   } # referer counting 
   if ( $h->{'ADBUY'} ) {            # by adbuy code
      $ses_adb{$h->{'ADBUY'}}=0 if ( ! $ses_adb{$h->{'ADBUY'}} );
      $ses_adb{$h->{'ADBUY'}}++ if ( $ses_adb{$h->{'ADBUY'}} );
   } # adbuy counting 
   if ( $h->{'ARTWORK'} ) {          # by banner ad artwork
      $ses_art{$h->{'ARTWORK'}}=0 if ( ! $ses_art{$h->{'ARTWORK'}} );
      $ses_art{$h->{'ARTWORK'}}++ if ( $ses_art{$h->{'ARTWORK'}} );
   } # artwork counting 
   if ( $h->{'BROWSER'} ) {          # by customer browser
      $ses_bro{$h->{'BROWSER'}}=0 if ( ! $ses_bro{$h->{'BROWSER'}} );
      $ses_bro{$h->{'BROWSER'}}++ #if ( $ses_bro{$h->{'BROWSER'}} );
   } # artwork counting 
   ## TBD - add affiliate counting
} # done with sessions

## start in on pageviews
#
$q="SELECT \* from clicks where CDATE>=$time_i and CDATE<$time_f ";
$sth = $dbh->prepare( $q );
$sth->execute;
$num=$sth->rows;
$tot{'pageviews_raw'}=$num;       # total pageviews burned through
$tot{'pageviews_sid'}=0;          # pageviews with session id known
foreach ( 1..$num ) {
   $h = $sth->fetchrow_hashref;
   next if ( ! $h->{'SID'} ); 
   $tot{'pageviews_sid'}++;   
   $session{$h->{'SID'}}->{'clicks'}=0 if (!$session{$h->{'SID'}}->{'clicks'});

   $session{$h->{'SID'}}->{'clicks'}++;
} # done with pageviews


## start in on orders
#
$q="SELECT \* from orders where CDATE>=$time_i and CDATE<$time_f ";
$sth = $dbh->prepare( $q );
$sth->execute;
$num=$sth->rows;
$tot{'orders_num'}=$num;       # total checkout entries (NOT finished orders)
$tot{'orders_act'}=0;          # actual count of real orders
$tot{'orders_tax'}=0;          # total sales tax collected
$tot{'orders_shp'}=0;          # total shipping paid
$tot{'orders_mds'}=0;          # total merchandise sold
$tot{'orders_tot'}=0;          # total charged for orders
foreach ( 1..$num ) {
   $h = $sth->fetchrow_hashref;

   ## TBD - exclude uncompleted orders

   ## count revenue for the session
   #
   $session{$h->{'SID'}}->{'revenue'}=$h->{'ORDTOTAL'};

   $tot{'orders_tax'}+= $h->{'ST_TAX'};
   $tot{'orders_shp'}+= $h->{'SHIPCOST'};
   $tot{'orders_mds'}+= $h->{'ST_TAXABLE'} + $h->{'ST_NONTAX'};
   $tot{'orders_tot'}+= $h->{'ORDTOTAL'};
} # done with orders


## start in on order lines
#
$q="SELECT \* from ocart where CDATE>=$time_i and CDATE<$time_f ";
$sth = $dbh->prepare( $q );
$sth->execute;
$num=$sth->rows;
$tot{'orders_raw'}=$num;       # total pageviews burned through

$tot{'orders_sid'}=0;          # pageviews with session id known
foreach ( 1..$num ) {
   $h = $sth->fetchrow_hashref;
   next if ( ! $h->{'SID'} ); 

} # done with order line items


## Output something...
#
$fromtime = scalar localtime($time_i);
$totime = scalar localtime($time_f);

print "Output for $fromtime to $totime\n";
$totvisitors = $ses_newu{'TOTAL'} + $ses_oldu{'TOTAL'};
print "New Visitors:$ses_newu{'TOTAL'}\tOld Visitors:$ses_oldu{'TOTAL'}\tTotal Visitors:$totvisitors\n";

print "REFERER\n";
foreach $k ( sort keys %ses_ref ) {
   print "$k\t$ses_ref{$k}\n";
} # done with REFERER
print "ADBUY\n";
foreach $k (sort keys %ses_adb ) {
   print "$k\t$ses_adb{$k}\n";
} # done with ADBUY

print "ARTWORK\n";
foreach $k (sort keys %ses_art ) {
   print "$k\t$ses_artb{$k}\n";
} # done with ARTWORK
print "BROWSERS\n";
foreach $k ( sort keys %ses_bro) {
   print "$k\t$ses_bro{$k}\n";
}
print "TOTALS\n";
foreach $k ( sort keys %tot ) {
   print "$k\t$tot{$k}\n";
} # done with totals
if ($totvisitors){

$pageviewpervis = $tot{'pageviews_raw'} / $totvisitors;
print "pageview per visitor\t$pageviewpervis\n";

$convrate =  $tot{'orders_num'} / $totvisitors;
print "conversion rate\t$convrate\n";
}
else
{
print "no visitors today\n";
}



}

$sth->finish if ( $sth );
$dbh->disconnect if ( $dbh );

## Output HTML
#



1;
