#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# sitestats.pl
#
# Program to compute daily statistics
#    RF 990108
#
# Version 0.7.3 IB 991115
#    minor cleanup
# Version 0.7.2 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991110
#    corrected the database call 
# Version 1 RF 990108
#     
$started=(times)[0];

## determine path from store root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$storeshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $storeshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $storeshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

use DBI;
require "$relpath/conf/store.pm";
require "$relpath/lib/state.pm";

## establish store-specific parameters
#
$site=setstore();

$store_ccart_ttl=3800;          # number of sconds before ccart->scart

## establish connection in
#
$s = "DBI:$site->{'datadriver'}:$site->{'table_state'}:$site->{'table_stat_serv'}:$site->{'table_stat_port'}";
$dbs = DBI->connect($s, $store_dbiuser, $store_dbipwd );

$now=time();                      # now

## calculate time of last midnight
#
( $s,$m,$h ) = (localtime($now))[0..2];
$lastmn = $now - ( $h*3600 + $m*60 + $s );


########### general flow
##  go through sessions for a 24 hour period
##     increment session counter by adbuy
##     increment session integrated duration by adbuy
##  go through orders for 24 hour period
##     increment order counter by adbuy
##     increment value counter by adbuy
##  calculate traffic by source, conversion efficiency by source
##
##
#

## this is a loop...

$thisday=1;
$maxback=30;

while ( $thisday < $maxback ) {
   ## temporal boundaries
   $time_start = $lastmn - ( 3600 * 24 * $thisday ); 
   $time_end   = $time_start + ( 3600 * 24 );

   @temp=localtime($time_start);
   $temp[4]++;                          # increment the month
   $display_time = "Day starting 19$temp[5] m$temp[4] d$temp[3] h$temp[2]\n";  

   ## 
   %ses_ct=();      # session counter by source
   %ses_br=();      # session counter by browser
   %ses_ti=();      # session integrated duration by source
   %ses_ui=();      # session ids that have known UIDs
   %ord_ct=();      # order counter by source
   %ord_va=();      # order value by source
   
   $q = 'SELECT * from sessions where ';
   $q.="CDATE>$time_start and CDATE<$time_end ";
   $sts = $dbs->prepare( $q );
   $sts->execute;
   $num=$sts->rows;
   foreach ( 1..$num ) {
      $h = $sts->fetchrow_hashref;
      ## set fields to z_other if blank
      $h->{'ADBUY'}="none" if ( ! $h->{'ADBUY'} );
      $h->{'REFERER'}="none" if ( ! $h->{'REFERER'} );
      $h->{'BROWSER'}="unknown" if ( ! $h->{'BROWSER'} );

      ## start adding to hashes..
      #
      if ( exists($ses_ct{$h->{'ADBUY'}}) ) {
         $ses_ct{$h->{'ADBUY'}}++;
         $ses_ui{$h->{'ADBUY'}}++ if ( $h->{'UID'} );
         $ses_ti{$h->{'ADBUY'}}+= ( $h->{'MDATE'} - $h->{'CDATE'} );
      } else {
         $ses_ct{$h->{'ADBUY'}}=1;
         $ord_ct{$h->{'ADBUY'}}=0;
         $ord_va{$h->{'ADBUY'}}=0;
         $ses_ui{$h->{'ADBUY'}}=0;
         $ses_ui{$h->{'ADBUY'}}=1 if ( $h->{'UID'} );
         $ses_ti{$h->{'ADBUY'}}= ( $h->{'MDATE'} - $h->{'CDATE'} );
      } # done with sessions by source
      if ( exists($ses_br{$h->{'BROWSER'}}) ) {
         $ses_br{$h->{'BROWSER'}}++;
      } else {
         $ses_br{$h->{'BROWSER'}}=1;
      } # done with browser counting

   } # done running through sessions

   ## do orders now...
   $q = 'SELECT * from orders where ';
   $q.="CDATE>$time_start and CDATE<$time_end and STATUS='pl' ";
   $sts = $dbs->prepare( $q );
   $sts->execute;
   $num=$sts->rows;
   foreach ( 1..$num ) {
      $h = $sts->fetchrow_hashref;
      $q = "SELECT SID,UID,ADBUY,REFERER from sessions where SID=$h->{'SID'} ";
      $sts2 = $dbs->prepare( $q );
      $sts2->execute;
      if ( $sts2->rows == 1 ) {
         $h2=$sts2->fetchrow_hashref;
         $h2->{'ADBUY'}="none" if ( ! $h2->{'ADBUY'} );
         $ord_ct{$h2->{'ADBUY'}}++;
         $ord_va{$h2->{'ADBUY'}}+=$h->{'ORDTOTAL'};
      } # no else...
   } # done checking orders

   $thisday++;
} # done looping for each day


$started=(times)[0] - $started;

$sts->finish if ( $sts );
$sts2->finish if ( $sts2 );
$dbs->disconnect if ( $dbs );

1;
