#!/usr/bin/perl -w
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# filetools.pm
#
# Version 20 IB 991115
#    mysql to dbi
# Version 19 IB 991114
#    corrected database cleanup
# Version 18 IB 991110
#    corrected database call
# Version 17 SH 990708
#    done system 6 retrofitting
#    changed queries and database connections
# Version 16 SH 990615
#    modified loadtemplate to default to "openmerchant" template files  
#    eliminated loadtemplate's check in the all directory
# Version 15 RF 990428
#    DISABLED comment removal from loadtemplate pending javascript detect
# Version 14 RF 990208
#    fixed use of fsrelativeroot and storecode -> store_sku_code
# Version 13 RF 981219
#    fixed errorlog streaming errors
# Version 12 RF 981211
#    fixed loadskutohashref
# Version 11 RF 981209
#    added loadskutohashref(sku):\% - returns all fiels of sku (+variant)
# Version 10 RF 981205
#    modified loadtemplate to look in store bits, then all/bits
# Version 9 RF 981127
#    bugfix on loadskutohash 
# Version 8 RF 981125
#    expanded loadtemplate to handle .sel files
# Version 7 RF 981122
#    fixed loadtemplate - it looks for an "all" template
#       if it cannot find the $storeshortname file...
#    added deterministic variant in loadskutohash
# Version 6 RF 981120
#    fixed undesried kill of include statements
# Version 5 RF 981112
#    modified permuteclean to kill self-referencing links if second arg is filename
#    modified permutebit to handle replacements with leading / and 
#       trailing _ characters
#    modified loadtemplate to handle include files as well, prefix i_
# Version 4 RF 981107
#    added function to read template vars and data
#    added readtemplate - reads file into string
#    ADDED these functions:
#       loadskutohash($sku) - \%h
#       loadtemplate($filename w/o extension, $bitspath) - $s
#       permutebit( $templatestring, $hashref ) - $s
#       permuteclean( $templatestring ) - $s 
#
# Version 3 RF 981022
#    added 
# Version 2 RF 981021
#    added hashthefile(targetpathfromroot,templatetouse);
#
# Library of functions for handling flat files
#
###############################################
# File input tools:
#    permute_file(filename,argument/value pairs) - reads 
#       the specified file from the filesystem and then
#       replaces the specified nametags with the specified
#       values, dumping final results to stdout
#
###############################################
#

## loadskutohashref(sku):\% - takes a sku [, variant] and returns all
#     RF 981209
#
sub loadskutohashref {
   my( $h,$dbh,$sth,$q,$hv,$sku,$var,@temp,$num );
   ( $sku,$var )=@_;
   $loadallvars=1;      # default to load all variants
   if ( length($var)!=2 ) {
      $loadallvars=0;
      $var="--";
   } #/end of whether or not to load all variants

   print FH "LOADSKUTOHASHREF got $sku $_[0] var $var $_[1] \n";

   $h={};
   if ( length($sku)!=8 ) {
      print FH "  ERROR - loadskutohashref exiting early - $sku - $var \n";
      return $h;
   } # end of short ciruit logic

   ## establish connection into tables
   $s="DBI:$site->{'datadriver'}:$site->{'table'}:$site->{'table_serv'}:$site->{'table_port'}";
   $dbh=DBI->connect($s, $site->{'store_dbiuser'}, $site->{'store_dbipwd'} );

   ## start loading fields
   #
   $q = 'SELECT * from things where SKU=';
   $q.= "'$sku' ";
   # print FH "Entering LOADSKUTOHASHREF \n  load $q\n";
   $sth = $dbh->prepare( $q );
   $sth->execute;
   $num=$sth->rows;
   if ( $num==1 ) {
      $h =  $sth->fetchrow_hashref ;
      #@temp=keys %{$h};
      #print FH "   loaded $sku with ", $#temp ," keys \n";
   } # load only if single row...
   if ( $num != 1 ) {  # exit early...
      $sth->finish if ( $sth );
      $dbh->disconnect if ( $dbh );
      return $h;
   } # done with short circuit if nothing on this sku ...

   ## start loading variant(s)
   #
   ## NOTE -- this is the single variant branch
   #          of the killed single/multi tree
      $q = 'SELECT * from variants where ';
      $q.= "SKU='$sku' and VCODE='$var' ";
      print FH "LOADSKUTOHASHREF: $q\n";
      $sth = $dbh->prepare( $q );
      $sth->execute;
      print FH "LOADSKUTOHASHREF: returned $sth->rows variants\n";
      if ( $sth->rows==1 ) {
         $hv = $sth->fetchrow_hashref;
         foreach $kv ( keys %{$hv} ) {
            $h->{$kv}=$hv->{$kv} if ( ! defined($h->{$kv}) ); 
         }# done loading each new variant key
      } # load only if single row...
   ## /NOTE done with branch logic for loading all variants or just one

   ## load the extracopy fields
   #
   $q="SELECT OBJID,BODY,FIELDNAME from extraprop where OBJID='$sku' ";
   $sth = $dbh->prepare( $q );
   $sth->execute;
   foreach ( 1..$sth->rows ) {
      @temp = $sth->fetchrow;
      $h->{$temp[2]}=string_exdbi($temp[1])  if ( ! $h->{$temp[2]} ); 
   }# done loading each extracopy field

   ## cleanup and return value
   #
   $sth->finish if ( $sth );
   $dbh->disconnect if ( $dbh );

   #print FH "LOADSKUTOHASHREF Loading $sku \n";
   foreach ( sort keys %{$h} ) {
      #print FH "$_ is $h->{$_} \n";
   }


   return $h;
} #/loadskutohashref


# loadtemplate - reads template file into string
sub loadtemplate {
   ## takes a filename (w/o path or extension) and bitspath, locate/reads file
   #    also strips comments from the file
   #    NOTE - SELECT pregeneration files are passed with their .sel extensions
   #    NOTE - if there's no template w/ the store's shortname, defaults to openmerchant 
   #
   my($f,$p,$s,$ok,$fr,$i);
   local(*FH);
   $f=$_[0];
   $p=$_[1];
   @g= split /-/,$f;
   $g="openmerchant";
   for ($i=1; $i <= $#g; $i++) {
       $g=$g."-".$g[$i];
   }
   # subdirs are "pages", "cells", "rows", "subcells","sel"
   $ok=0;   
   if ( $f =~ /\.sel$/ ) {
      # it is a .sel file
      $fr="$p/sel/$f";
      if ( -e "$fr" ) {
         $ok=1;
      } else { # look at openmerchant
         $fr="$p/sel/$g";
         $ok=1 if ( -e "$fr" );
      }
   } elsif ( $f =~ /^cell/ ) {
      # it is a cell
      $fr="$p/cells/$f.txt";
      if ( -e "$fr" ) {
         $ok=1;
      } else { # look at openmerchant
         $fr="$p/cells/$g.txt";
         $ok=1 if ( -e "$fr" );
      }
   } elsif ( $f =~ /^(subc|sc)/ ) {
      # subcell
      $fr="$p/subcells/$f.txt";
      if ( -e "$fr" ) {
         $ok=1;
      } else { # look at openmerchant
         $fr="$p/subcells/$g.txt";
         $ok=1 if ( -e "$fr" );
      }
   } elsif ( $f =~ /^row/ ) {
      # row
      $fr="$p/rows/$f.txt";
      if ( -e "$fr" ) {
         $ok=1;
      } else { # look at openmerchant
         $fr="$p/rows/$g.txt";
         $ok=1 if ( -e "$fr" );
      }
   } elsif ( $f =~ /^inc_/ ) {
      # include
      $fr="$p/include/$f.html";
      if ( -e "$fr" ) {
         $ok=1;
      } else { # look at openmerchant
         $fr="$p/include/$g.html";
         $ok=1 if ( -e "$fr" );
      }
   } else {
      # default guess - page template
      $fr="$p/pages/$f.txt";
      if ( -e "$fr" ) {
         $ok=1;
      } else { # look at openmerchant
         $fr="$p/pages/$g.txt";
         $ok=1 if ( -e "$fr" );
      }
   }
   $ok=1 if ( -e "$fr" );
   ## if that failed, try includes
   #
   if ( ! $ok ) {
      $fr="$p/include/$f.html";
      if ( -e "$fr" ) {
         $ok=1;
      } else { # look at openmerchant
         $fr="$p/include/$g.html";
         $ok=1 if ( -e "$fr" );
      }
   }
   ## if that failed, try the $p directory itself  
   #
   if ( ! $ok ) {
      $fr="$p/$f.txt";
      $ok=1 if ( -e "$fr" );
      if ( ! $ok ) {
         $fr="$p/$g.txt";
         $ok=1 if ( -e "$fr" );
      }
   }
  

   ## if OK, get the file, otherwise return an error
   if ( $ok ) {
      # success
      open(FH, "<$fr");
      $s = join "", (<FH>);
      close(FH);
   } else {
      $s="f=$f <br> g=$g <br> p=$p \n";
   }

   ## now remove all html comments from the template, EXCEPT includes
   #
   # $s =~ s/<!--(?!(#include|#exec|# include|# exec))(.+?)-->//gsm ;

   return $s;
} #/loadtemplate

## loadskutohash - takes a sku, returns a hashref
#
sub loadskutohash {
   my($sku,$sth,$q,$h,$i,$f,@temp);
   $sku=$_[0];
   
   ## primary sku fields
   #
   $q = "SELECT * from things where SKU='$sku'";
   $sth = $dbh->prepare( $q );
   $sth->execute;
   if ( $sth->rows == 1 ) {
      $h = $sth->fetchrow_hashref;
   } else {
      $h = {};
   }
   $sth->finish;

   ## virtual sku fields
   #
   $q = "SELECT OBJID,FIELDNAME,BODY from extraprop where OBJID='$sku'";
   $sth = $dbh->prepare( $q );
   $sth->execute;
   if ( $sth->rows ) {
      ## there is extracopy
      #
      for ($i=0; $i < $sth->rows; $i++ ) {
         ( $sku, $f, $q ) = $sth->fetchrow_array;
         $h->{$f} =  $q; 
      } 
   }
   $sth->finish;

   @temp=keys %{$h};
 
   # remove dbi-safe encodings
   while ( @temp ) {
      $k = shift @temp;
      $h->{$k}=string_exdbi( $h->{$k} );
   } # end of de-dbi-safe-encoding strings

   return $h;

} #/loadskutohash

## listrowkinds - takes a template string, returns array or rownames
#
sub listrowkinds {
   my(@kinds);

   @kinds=();

   return @kinds;

} #/listrowkinds


## permutebit - takes a template string, and a hash ref
#             - returns a string, possibly with unsubbed vars
sub permutebit {
   local($s,$n,$v,$h);
   ( $s,$h ) = @_;
   if ( ! $h ) {
      return $s;
   } # 

   foreach ( keys %{$h} ) {
      $v="";
      $v =  $h->{$_} if ( defined($h->{$_}) );
      $n = "__".$_."__";
      $s =~ s/$n/$v/gms ;
   }
   
   return $s;
} #/permutebit


## permuteclean - takes a template that has already undergone var expansion
#                 and cleans out remaining unexpanded variables
sub permuteclean {
   my($s,$f);
   $s = $_[0];
   $f="";
   $f=$_[1] if ( $#_>0 );    # kill links to this file, usually itself

   $s =~ s/(\s|[;:,<>()"^$?+.])__(\w|-|\.)+__(\s|[;:,<>()"^$?+.])/$1$3/g;

   if ( length($f)>7 ) {    # if there is a minimally plausible filename...
      $s =~ s/<A HREF(\S+?)($target)(.+?)>//gis ;
   }

   return $s;
} #/permuteclean


1;
