#/usr/bin/perl
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# formtools.pm
# 
# Set of form and page generation tools
#
#
# Version 0.7.7 IB 991112
#    adding subroutines string_todbi and string_exdbi to the string_tomysql
#    and string_exmysql
# Version 0.7.6 IB 991110
#    minor cleanup
# Version 0.7.5 RF 990830
#    made string_exmysql sensitive to \\_ stuff
# Version 0.7.4 RF 990830
#    fixed bug in string_tomysql for "_"->"\_"
# Version 0.7.3 RF 990830
#    fixed bug in string_exmysql for "\_"->"_"
# Version 0.7.2 RF 990829
#    added variant sensitivity to sortlinks()
# Version 0.7.1 GF 990829
#    removing quotes from selects in gen_select
# Version 9 SH 990819
#    hack fix of note on sortlinks
# SH Note:  sortlinks needs to be rewritten 
# Version 8 RF 990208
#    corrected use of fsrelativeroot and storecode -> store_sku_code
# Version 7 RF 981211
#    added nicedecimals( $s ):$s - evens up trailing zeros to currency
# Version 6 RF 981208
#    added getformrows( \% ):\% - returns hashref of hashrefs of rows
# Version 5 RF 981127
#    added sortlinks
#    fixed gen_select matching bug
# Version 4 RF 981125
#    modified keylength operator from equality to >= in gen_select
#    added make_hash_htmlsafe - html encodes certain fields
# Version 3 RF 981123
#    added gen_safeselect - generate select w/only one choice
# Version 2 RF 981013
#    added string to/from mysql conversion operations
#    added string to/from html conversion operations
# version 1 RF 981009
#
# 

#
# gen_select - generates an OPTION value [SELECTED] pulldown
#              content when passed "default","key-length","key-value pairs 
#              vertical slash delimited array"
#           
        

## nicedecimals( $s ) : $s;
#
sub nicedecimals {
   my( $x,$y,$a,$b,$r,$i );
   $i=$_[0];

   ( $left,$right ) = split /\./, $i;
   $left="0" if (!$left);
   $right="00" if (!$right);
   $right.="0" if (length($right)==1);
   $right = substr $right, 0, 2;
   return "$left.$right";
   
} #/nicedecimals


## getformrows( \% ): \%->\%  takes a hashref, finds rows
#     and constructs output rows. FORM KEYS for rows are DELETED
#
sub getformrows {
   my( $r,$k,$v,$f,@temp,$rows );
   $f=$_[0];

   $rows={};

   @temp=keys %{$f};
   while (@temp) {
      $k=shift @temp;
      next if ( $k !~ /^row\./ );
      $v = $f->{$k};
      delete( $f->{$k} );
      $r = ( split /\./, $k )[1];    # rowid
      $k =~ s/^row\.$r\.// ;
      $rows->{$r}->{$k} = $v;
   } # done running through primary form data
   
   return $rows;
} #/getformrows


## sortlinks( @ ): @ - take a list of links and sort them 
#
# 0.7.2 - RF changed to support variants
#
sub sortlinks {
   my( @h,@o,@temp,$i,$k,$c );
   @h = @_; 
   @o = ();
   @temp=();
   for ( $i=0; $i<=$#h; $i++ ) {
      $k='0';
      $k=$h[$i]->{'LINKWT_USE'};
      $k=$h[$i]->{'LINKWT_MAN'} if ( ( ! $k ) and ( $h[$i]->{'LINKWT_MAN'} ) );
      $k=$h[$i]->{'LINKWT_SYS'} if ( ( ! $k ) and ( $h[$i]->{'LINKWT_SYS'} ) );
      $k=$h[$i]->{'LINKWT_USE_inp'} if ( ( ! $k ) and ( $h[$i]->{'LINKWT_USE_inp'} ) );
      $k='10' if ( ! $k );

      # $c=$h[$i]->{'CELLWIDTH'};   # do not want cellwidth right now
      
      $c='0' if ( ! $c );
      # now construct sort key
      $k = "$c".'.'."$k X$i";
      push @temp, $k; 
   } # end of running through key creation for sorting

   sub bywidthweight { $b <=> $a; } # sort numerically descending 
   @temp=sort bywidthweight @temp;

   while ( @temp ) {
      ( $k, $i ) = split /X/, shift @temp;
      push @o, $h[$i] ;
   } # end of assembling new array   
   return @o;
} #/sortlinks


##### make_hash_htmlsafe( \% ):\% - html encode hash entries matching
#        certain fieldnames
sub make_hash_htmlsafe {
   my(%h,@temp,$k,$v,$fieldlist);
   $fieldlist='(TITLE|ALTTITLE|DESCRIPT|BRIEF|KEYWORDS|AUTHOR|HANDLER)';
   %h = %{$_[0]};
   @temp = keys %h;
   while (@temp) {
      shift @temp;
      next if ( /_SELECT$/ ); 
      next if ( ! /$$fieldlist/ );    
      $h{$_} = string_tohtml( $h{$_} );
   } #/end of looling through keys

   return \%h;
} #/make_hash_htmlsafe


##### gen_safeselect - makes deterministic pulldowns
sub gen_safeselect {
   my(@list,$l,$k,$v,$x,$d,$base);
   ( $base,$l,@list ) = @_;
   $x="";
   $d=1;
   while (@list) {
      ( $k,$v )  = split /\|/, shift @list;
      if (( $base ) and ( $d ) and ( length($base)>=$l )) {
         if ( $base eq $k ) {
            $x.= "<OPTION VALUE=\"$k\" ";
            $x.="SELECTED $k = $base";
            $x.="> $v \n";
            $d=0;
         }
      }
   }
   return $x;
} # /gen_safeselect


##### Gen_Select
sub gen_select {
   my(@list,$l,$k,$v,$x,$d,$base);
   local(*FH);
   ( $base,$l,@list ) = @_;

   $x="";
   $d=1;
   while (@list) {
      ( $k,$v )  = split /\|/, shift @list;
       $x.= "<OPTION VALUE=$k ";  # GF removed quotes

      if (( $base ) and ( $d ) and ( length($base)>=$l )) {
         if ( "$base" eq "$k" ) {
            $x.="SELECTED $k = $base";
            $d=0;
         }
      }
      $x.="> $v \n";
   }
   return $x;
} # /gen_select


sub myhomegen_select {
   my(@list,$l,$k,$v,$x,$d,$base, $m);
   local(*FH);
   ( $base,$l,@list ) = @_;

   $x="";
   $d=1;
   while (@list) {
      ( $k,$v )  = split /\|/, shift @list;
      $m=$k." | ".$v;
      $x.= "<OPTION VALUE=\"$k\" ";
      if (( $base ) and ( $d ) and ( length($base)>=$l )) {
         if ( "$base" eq "$k" ) {
            $x.="SELECTED $k = $base";
            $d=0;
         }
      }
      $x.="> $m \n";
   }
   return $x;
} # /gen_select


##### string_tohtml

sub string_tohtml {
   local( $x );

   $x = $_[0];
   $x =~ s/"/&quot;/g;
   
   return $x;
} # /string_tohtml


##### string_exhtml

sub string_exhtml {
   local( $x );

   $x = $_[0];
   $x =~ s/&quot;/"/g;
   
   return $x;
} # /string_exhtml


##### string_tomysql
#
# 0.7.4-990830 RF - changed to single pass 
#
sub string_tomysql {
   local( $x );

   $x = $_[0];
   $x =~ s/('|"|%|_)/\\$1/g ;

   return $x;
} # /string_tomysql


##### string_exmysql
#
# 0.7.5-990830 RF - added preliminary pass to kill \\_ stuff
# 0.7.3-990830 RF - changed to single pass substitution
#
sub string_exmysql {
   local( $x );

   $x = $_[0];
   $x =~ s/\\\\('|"|%|_|t|n|r)/$1/g ;         # catches asymetric encoding
   $x =~ s/\\('|"|%|_|t|n|r)/$1/g ;

   return $x;
} # /string_exmysql


##### string_todbi
#
# 0.7.7-991112 IB - added string_todbi to eventually replace string_tomysql
#
sub string_todbi {
   local( $x );

   $x = $_[0];
   $x =~ s/('|"|%|_)/\\$1/g ;

   return $x;
} # /string_todbi


##### string_exdbi
#
# 0.7.7-991112 IB - added string_exdbi to eventually replace string_exmysql 
#
sub string_exdbi {
   local( $x );

   $x = $_[0];
   $x =~ s/\\\\('|"|%|_|t|n|r)/$1/g ;         # catches asymetric encoding
   $x =~ s/\\('|"|%|_|t|n|r)/$1/g ;

   return $x;
} # /string_exdbi


1;
