#!/usr/bin/perl -w
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# login.pm
#
# Version 0.7.3 IB 991205
#    corrected a database call
# Version 0.7.2 IB 991115
#    testmode, database cleanup
# Version 0.7.1 IB 991110
#    corrected a database call
# Version 6 SH 990708
#    totally retrofitted to system 6
#    got rid of adminsessions, now queries in sessions
# Version 5 SH 990618
#    changed the table lookup from table_customer to table
# Version 4 RF 981219
#    converted to store.pm multiple server compliance
# Version 2 RF 981122
#    check_login now uses globals for connect data
# Version 1 RF 981026
#
# Library of functions for handling login validation
#
###############################################
# Login Validation:
#    check_login 
#       invoked after the form data read but before sending
#       out the http header (it may spew a redirect to the
#       login script).
#
###############################################
#

# newsid
sub newsid {
   ## return a new unique session id
   #
   # TBD - sid assignment mechanism
   #
   local($sid);

   $sid=time();

   return $sid;
} # /newsid

# check_login
sub check_login {
   # takes script, mode, SID as args
   # returns $USERNAME if sid for this script and mode, otherwise ""
   #
   local($sc,$m,$sd,$rv,$sti,$num,$rs,$life,$uid,$s,$q,$v,$n,$connect);
   ( $sc,$m,$sd ) = @_;

   $uid="";

   $rv=1;   # ok by default
   $rs="";
   $life=3700;  # number of seconds to live... 

   ## check for a non-zero sid  
   if ( length("$sd")<2 ) { 
      $rv=0;
      $rs="ERROR - there is no session ID $sd";
   }

   if ( $rv ) {
      $connect="DBI:$site->{'datadriver'}:$site->{'table'}:$site->{'table_serv'}:$site->{'table_port'}";
      $dbu = DBI->connect($connect,$site->{'store_dbiuser'},$site->{'store_dbipwd'});
      
      $q = "SELECT UID,SID,MDATE from sessions where SID='$sd' ";
      $sti = $dbu->prepare( $q ); 
      $sti->execute;
      $num=$sti->rows;
      if ( $num == 1 ) {
         # OK, have one sid matching
         %row = %{$sti->fetchrow_hashref};
         $dt=time() - $row{'MDATE'};
         if (  $dt>$life ) {
            # login has expired
            $rv=0;
            $rs="ERROR - user logged out! login inactive more than $life seconds!";
            # remove dead login from table
            $q="DELETE from sessions where SID='$sd' ";
            $sti = $dbu->do($q);
            #/checking MDATE 
         } else {
            ## YES - there is exactly one SID for this...
            # update the time of last click
            $num=time();
            $uid=$row{'UID'};
            $q="UPDATE sessions SET MDATE=$num where SID='$sd' ";
            $sti = $dbu->do($q);
         } # done checking freshness of login...
      } else {
         # too few or too many records
         if ( $num ) {
            %row = %{$sti->fetchrow_hashref};
            $rs="ERROR - there are too many records for $sd";
            # remove bad records for SID
            $q="DELETE from sessions where SID='$sd'";
            $sti = $dbu->do($q);
         } else {
            $rs="ERROR - $sd bad sid! you must log into the system!";
         }
         $rv=0;
      }
   } #/looking for sid
 
   ## now test the uid permissions 
   #
   if ($rv) {
      ## get the user record
      #
      $q = "SELECT UID,UPERM,MDATE,UNAME from contacts where UID='$uid' ";
      $sti = $dbu->prepare( $q ); 
      $sti->execute;
      $num = $sti->rows;
      ## verify exactly one row returned
      #
      if ( $num==1 ) {
         %row=%{$sti->fetchrow_hashref};
         ## now check permissions
         #
         @perm=split /;/, $row{'UPERM'};
         while (@perm) {
            $s=shift @perm;
            ($n,$v) = split /:/, $s;
            if ( $n eq "all" ) {
               ## the "God" user
               #
               $rs="$row{'UNAME'}";
               @perm=();
            } elsif ( $n =~ /^$s/ ) {
               ## this is the section on this script
               #
               if ( $v =~ /($m|^all)/ ) {
                  $rs="$row{'UNAME'}";
                  @perm=();
               } else {
                  $rs="Error - $row{'UNAME'} lacks access to $m";
                  $rv=1;    # yes, leave true
                  @perm=();
               } # done checking user perm in this script
            } else {
               ## no match...
               #  nothing to do...
            } 
         } # done with user permissions list
         ## now see if $rs defined
         #
         if (  length($rs) == 0 ) {
            $rs = "Error - insufficient permissions for $row{'UNAME'} to do $m ";
         } # done testing length of message string
      } else {
         ## was expecting different number of rows
         # unable to verify permissions, send error message
         $rv=0;
         $rs="ERROR - user id $uid has $num rows in adminusers! Contact Administrator!";
      } #/end testing on number of user records in system
   } #/testing user permissions

   ## test whether to redirect back to login screen or not 
   #
   if ( ! $rv ) {
      ## redirect them
      $x = (split /\?/, $ENV{'REQUEST_URI'})[0];  # remove GET stuff
      @temp=split /\//, $x;                       # 
      pop @temp;                                  # throw away scriptname
      shift(@temp) if ( ! $temp[0] );
      $x= join "/", @temp;                         
      $scriptpath="$ENV{'HTTP_HOST'}/$x/login.cgi";
      print "Location:  http://$scriptpath\n\n"; 
   }

   if ( length($rs)<1 ) {
      $rs = "ERROR - unknown reason for lack of error message at login";
   }

   $dbu->disconnect if ($dbu );

   return $rs;
}
# /check_login


1;
