#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# xml.pm
#
# Functions to create/parse xml files
#    C/B RF 26 August 1999
#
# BUGS:
#
# TBD:
#
# AUTHORS:
#    RF - Rob Ferber
#
# VERSION HISTORY:
# Version 0.7.4 - 991112 IB
#    Correcting a call to database
# Version 0.7.3 - 990921 RF
#    fixed template load error
# Version 0.7.2 - 990907 RF
#    added function:
#       xml_make_bod56( $s ): $s - takes on orderid and exports
#                             a single multiline string xml 
#                             BOD36 for output however
# Version 0.7.1 - 990826 RF
#    created library with functions as follow:
#       xml_clean( $s ) - cleans up unreplaced variables
#       xml_loadtemplate(objectname,path) - loads XML template and parses
#       xml_permute( $s, \%h ) - permutes template $s with hash %h
#
#


########### FUNCTIONS ############
#

## xml_clean( $s ):$s 
#     Takes a template as string and removes unreplaced variables
#     Returns a string
#
#     Created with lib 0.7.1 990826 RF
#     Version 0 - 0.7.1 - 990826 RF
#
sub xml_clean {
   # ( $s ):$s
   my($i,@blanks);
   $i=$_[0];

   ## TBD - real replacement pattern
   @blanks = ( $i =~ s/__x__//gs );

   return $i;
} #/xml_clean


## xml_loadtemplate( $t,$p ):\%h
#     Takes a template target name and store path
#     Returns a hashref:
#        {'ok'}   = 1 for successful load, 0 for failure
#        {'error'}= any error messages
#        {'name'} = the template name
#        {'fname'}= the template full filename
#        {'skel'} = the template as a string
#        {'vars'} = comma separated list of variables found
#        {'fx'}   = comma separated list of functions called
#        {'flds'} = comma separated list of fields found
#
#     Created with lib 0.7.1 990826 RF
#     Version 0 - 0.7.1 - 990826 RF
#
sub xml_loadtemplate {
   my($t,$p,$h,@temp);
   $t=$_[0];          # the target name
   $p=$_[1];          # the root path to the store
 
   $h={};             # create a blank hashref for output
   $h->{'name'}=$t;   # requested target 
   $h->{'ok'}=0;      # default assume operation failed

   ## start trying to locate template file
   #    NOTE - all template files should be $store/bits/admin/xml/
   #           uppercase filenames with lowercase ".txt" extensions
   #
   $h->{'fname'}="$p/bits/admin/xml/$t.txt";    # construct filename
   if ( -f "$h->{'fname'}" ) {                  # if the file exists
      ## open and load the file
      #
      open( TEMPLATE, "<$h->{'fname'}" );
      $h->{'skel'} = join "", ( <TEMPLATE> );
      close( TEMPLATE );

      ## parse and preprocess the file
      $h->{'vars'}="";
      $h->{'fx'}="";
      $h->{'flds'}="";

      ## from filetools.pm
      #@temp = (
      #   $h->{'skel'} =~ 
      #   /(\s|[;,<>()"^$?+.])__(\w|-|\.|\:)+__(\s|[;,<>()"^$?+.])/gs
      #);

      @temp = (
         $h->{'skel'} =~ 
         /(__[\w:]+__)/g
      );

      foreach $k ( @temp ) {

         $h->{'vars'}.="," if ( $h->{'vars'} );
         $h->{'vars'}.=$k;
         ## TBD - field and function lists...
         #

      } # done with each substitute-able parameter 
       
   } else {                                     # file not found
      $h->{'error'}="Error - $t not found in $h->{'fname'}!";
      $h->{'skel'}="";
      $h->{'vars'}="";
      $h->{'fx'}="";
      $h->{'flds'}="";
   } # done with locate template or fail...

   return $h;
} #/xml_loadtemplate


## xml_make_bod36( $s ):$s
#     Takes an order number and returns a finished XML BOD36 document
#     NEEDS GLOBAL store environment data
#     CALLS other subs within this library
#
sub xml_make_bod36 {
   my($o,$s,$dbh,$sth,$num,$q,$h,$sa,$ba);
   $o=$_[0];                                    # order number
    
   ## error trapping
   return "ERROR - a valid orderid is required!\n" if ( ! $o );
 
   use DBI;
   $s = "DBI:$site->{'datadriver'}:$site->{'table'}:$site->{'table_serv'}:$site->{'table_port'}";
   $dbh = DBI->connect($s, $site->{'store_dbiuser'}, $site->{'store_dbipwd'} );
   
   $q = "SELECT \* from orders where OID='$o' ";
   $sth=$dbh->prepare( $q );
   $sth->execute;
   $num=$sth->rows;
   if ( $num ne 1 ) {
      ## cleanup connection and return "bad record count"
      #
      $sth->finish;
      $dbh->disconnect;
      return "ERROR - $num records found for order $o\n";
   }
   
   ## must be exactly one record found
   #
   $h=$sth->fetchrow_hashref;

   ## now get the addresses referenced
   #
   


   $sth->finish;
   $dbh->disconnect;
  
   return $s;
} #/xml_make_bod36




## xml_permute( $s,\%p ):$s
#     Takes a template string and hashref
#     Returns a template string with variables expanded
#
#     Created with lib 0.7.1 990826 RF
#     Version 0 - 0.7.1 - 990826 RF
#
sub xml_permute {
   my($t,$k,$v,%nv);
   $t=$_[0];        # template string
   $nv = $_[1];     # hashref of name/value pairs

   foreach $k ( keys %{ $nv } ) {
      $v=$nv->{$k};
      $t =~ s/__($k)__/$v/gs ;
   } # done with global replacement

   return $t;
} #/xml_permute

1;
