#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999 OpenSales, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

##
#    import_shipping_tables.pl
#
# Script to import various format tables of shipping data 
# into zonemap and zonerate tables
#
# For present, contains a bunch of file parsers at the
# end of the script, and some variables to select filename
# and type for import.
#
# output is a file suitable for pipeing to, for example, mysql
#
# Version 0.7.4 IB 991115
#    GPL license
# Version 0.7.3 IB 991114
#    deleting some commented out statements
# Version 0.7.2 IB 991113
#    reduce or remove the dependencies on mysql
# Version 0.7.1 IB 991104
#    general cleanup
# Version 4 GF 990830
#    configuration added for myhome
# Version 4 RF 990527
#    USPS rate files import written
# Version 3 RF 990527
#    USPS zone files import written
# Version 2 RF 990525
#    UPS rate files import written
# Version 1 RF 990525
#    UPS zone file import operational

####
# Initialization
####

$now=time();

### Shipper that file is from
## 0=UPS, 1=US Postal Service, 2=Federal Express
$shipper=( "UPS","USPS","FedEx" )[2];

### Shipment "From" Zipcode
## 0=sample outbound zipcodes
$fromzip=( "91789" )[0];

### Reading map or rates?
## 0 = zone chart, 1 = rate chart
$filetype=( "zones","rates" )[1];

# construct the name of the file to dump
$output_file="$shipper.$filetype.$fromzip.dbid";

# from ground through next day AM method skus...
if ( $shipper eq "UPS" ) { 
   @methodskus=("xspm0001","xspm0002");
   # cheat on the input file
   $input_file="data/910.csv";        # use for single file
   $input_skel="data/UPS_rates_";               # use for series of files
} elsif ( $shipper eq "USPS" ) {
   @methodskus=("xspw0007","xspw0008");
   # cheat on the input file
   $input_file="data/USPS_from12212.html";       # use for single file
   $input_skel="data/USPS_rates_";               # use for series of files
} elsif ( $shipper eq "FedEx" ) {	
   @methodskus=("xspm0003");
   $input_file="data/fedex_91000.txt";
   $input_skel="data/FedEx_rates_";
} # done with available methods

@input_files=();

####
# Begin actual file processing
####

# initialize arrays
@infile=();
@outfile=();


if ( $filetype eq "zones" ) {           # zone table one file
   @input_files=( $input_file );
} else {                                # rate tables multiple files
   foreach $i ( @methodskus ) {
      $newfile = "$input_skel$i";
      $newfile.=".csv" if ( $shipper eq "UPS" );
      $newfile.=".txt" if ( $shipper eq "USPS" );
      $newfile.=".txt" if ( $shipper eq "FedEx" ); 
      push @input_files, ( $newfile );
   } # done building file list
} # done with monolithic or not...

foreach $input_file ( @input_files ) { 
   # read input file
   open(FH, "<$input_file" );
   @infile=(<FH>);
   chomp @infile;
   close(FH);
   next if ( ! @infile );

   print " $#infile input lines \n";
   @outincr=();

   # choose which parser and parse 

   if ( $filetype eq "zones" ) {
      if ( $shipper eq "UPS" ) {
         @outincr=parse_UPS_dom_zones(@infile);
      } elsif ( $shipper eq "USPS"  ) {
         @outincr=parse_USPS_dom_zones(@infile);
      } elsif ( $shipper eq "FedEx" ) {
	@outincr=parse_FedEx_dom_zones(@infile);
      } # done with shipper decision tree
   } else {                           # this is a rates batch
      if ( $shipper eq "UPS" ) {
         $l = $input_file;
         $l =~ s/^$input_skel// ;
         $l =~ s/\.csv$// ;
         $shipmeth=$l;
         @outincr=parse_UPS_dom_rates(@infile);
      } elsif ( $shipper eq "USPS"  ) {
         ## now have different parserS for Express and Priority...
         if ( $input_file =~ /xspw0007/ ) { # priority mail  
            $shipmeth="xspw0007";     
            @outincr=parse_USPS_dom_rates_priority(@infile);
         } else {                           # express mail   
            $shipmeth="xspw0008";     
         } # done with decision on which parser to use

      } elsif ( $shipper eq "FedEx" ) {
	  $l = $input_file;
	  $l =~ s/^$input_skel// ;
	  $l =~ s/\.txt$// ;
	  $shipmeth=$l;
        @outincr=parse_FedEx_dom_rates(@infile);
      }
   } # done with rates vs. zones branches
  
   push @outfile, @outincr;

} # done looping on input files

# write output file
open(FH, ">$output_file");
while ( @outfile ) {
   print FH shift @outfile;
} # done dumping output array
close(FH);

####
# Specific Parsers
####

### parse_USPS_domestic_rates_priority
##     takes an array of lines (file)
##     outputs an array of lines (for dump to file)
#
#      Priority Shipping rates table
#      new RF 4-990527
#
sub parse_USPS_dom_rates_priority {
   ## Globals used - $shipmeth for shipping sku
   my( @lines,@output,@special,@temp,@zones );
   @lines=@_;
   @output=();
   @special=();
   
   ## format of .csv file data segment is:
   #  weight, L-3,4,5,6,7,8
   #     Next Day Air Saver, Next Day Air
   
   while (( $lines[0] !~ /pounds/ ) and ( @lines )) {
      $l = shift @lines;
      print "$l \n";
   } # done chopping header lines
   $l = shift @lines;   # discard last header line
   
   ## now run through lines and generate insert statements
   #
   while ( @lines ) {
      $l = shift @lines;
      chop $l;
      $l =~ s/(\$| )//g ;
      @temp = split /\t/, $l;
      foreach $j ( 2..$#temp ) {   # catch zones 4-8
         $q = "INSERT INTO zonerate VALUES ";
         $q.= "('$shipmeth','$j',$temp[0],$temp[$j],0.00,'','',$now,'') ";
         $q.= " \\g \n";
         push @output, ( $q );
      } # done with priority zones 4-8   
      ## now handle zones 1,2,3
      foreach $j ( 1..3 ) {   # zones L-3 in column 1
         $q = "INSERT INTO zonerate VALUES ";
         $q.= "('$shipmeth','$j',$temp[0],$temp[1],0.00,'','',$now,'') ";
         $q.= " \\g \n";
         push @output, ( $q );
      } # done with priority zones 1-3

   } # done running through lines

   return @output;
} # end of parse_USPS_dom_rates_priority


### parse_USPS_domestic_zones
##     takes an array of lines (file)
##     outputs an array of lines (for dump to file)
#
sub parse_USPS_dom_zones {
   my( @lines,@output,@special,@temp);
   @lines=@_;
   @output=();
   @special=();
   
   ## format of .html file data segment is:
   #  <td>zip [.. zip]</td>
   #  <td>zone</td>

   ## preprocess for data segment
   #
   @temp=();
   while ( @lines ) {
      $l = shift @lines;
      next if ( $l !~ /<td>.+<\/td>/ );
      $l =~ s/(\s|\*|<td>|<\/td>)+//g ;
      if (length($l)>2) {    # zipcode block
         $l.=":";        
      } else {               #
         $l.="\n";
      } # done with zipcode or not      
      push @temp, ( $l );
   } # done chewing out relevant bits
   
   ## merge lines 
   #
   $l="";
   while ( @temp ) { 
      $li = shift @temp;
      if ( $li =~ /\n/ ) {
         $l.=$li;
         push @lines, ( $l );    
      } else {
         $l=$li;
      } # 
   } # done merging lines 
   
   ## start writing output file lines
   #
   while ( @lines ) {
      $l = shift @lines;
      chomp $l;
      ( $zips,$zone ) = split /:/, $l;
      ( $zmin,$zmax ) = split /\.\./, $zips;
      $zmax=$zmin if ( ! $zmax );
      $zmax.="99";
      $zmin.="00";

      $new = "INSERT INTO zonemap VALUES ";
      $new.="('xspw0007','$fromzip','USA',";
      $new.="'$zmin','$zmax','USA','$zone','',$now,'') \\g \n";
      push @temp, ( $new );

   } # done constructing insert statements

   ## add express domestic as one zone
   $new = "INSERT INTO zonemap VALUES ";
   $new.="('xspw0008','$fromzip','USA',";
   $new.="'00200','99999','USA','dom','',$now,'') \\g \n";
   push @temp, ( $new );

   return @temp;
} # done with parse_USPS_domestic_zones


### parse_FedEx_domestic_zones
##     takes an array of lines (file)
##     outputs an array of lines (for dump to file)
#
sub parse_FedEx_dom_zones {
   my( @lines,@output,@special,@temp);
   @lines=@_;
   @output=();
   @special=();
   
   ## format of .csv file data segment is:
   #  zip [-zip],Ground,3 Day Select,2nd Day Air,2nd Day Air AM,
   #     Next Day Air Saver, Next Day Air
   
   while (( $lines[0] !~ /^00/ ) and ( @lines )) {
      $l = shift @lines;
      print "$l \n";
   } # done chopping header lines

   # read through simple zipcodes
   while (( $lines[0] =~ /^\d\d\d/ ) and ( @lines )) {
      $l = shift @lines ;
      ($zmin,$zmax) = split /\-/, $l;
      $zone = shift @lines;
      
      $zmax=$zmin if ( ! $zmax );
        
      $new = "INSERT INTO zonemap VALUES ";
      $new.="('$methodskus[$i]','$fromzip','USA',";
      $new.="'$zmin','$zmax','USA','$zone','',$now,'') \\g \n";
         
      push @output, ( $new );
   } # done reading through the "normal" portion of the table

   return @output;

} # UPS_zones done with readthrough of file



### parse_UPS_domestic_zones
##     takes an array of lines (file)
##     outputs an array of lines (for dump to file)
#
sub parse_UPS_dom_zones {
   my( @lines,@output,@special,@temp);
   @lines=@_;
   @output=();
   @special=();
   
   ## format of .csv file data segment is:
   #  zip [-zip],Ground,3 Day Select,2nd Day Air,2nd Day Air AM,
   #     Next Day Air Saver, Next Day Air
   
   while (( $lines[0] !~ /^00/ ) and ( @lines )) {
      $l = shift @lines;
      print "$l \n";
   } # done chopping header lines

   # read through simple zipcodes
   while (( $lines[0] =~ /^\d\d\d/ ) and ( @lines )) {
      $l = shift @lines ;
      @temp = split /\,/, $l;
      $zr=shift @temp;
      ($zmin,$zmax) = split /\-/, $zr;
      $zmax=$zmin if ( ! $zmax );
      ## take from 3 digit prefixes to full zips 
      $zmin.="00";
      $zmax.="99";
      foreach $i ( 0..$#temp ) {
         next if ( $temp[$i] =~ /\-/ );      # skip null entries 
         next if ( ! $temp[$i] );            # skip null entries 
         next if ( $temp[$i] =~ /\[\d]/ );   # skip special cases
        
         # construct the insert statement 
         $new = "INSERT INTO zonemap VALUES ";
         $new.="('$methodskus[$i]','$fromzip','USA',";
         $new.="'$zmin','$zmax','USA','$temp[$i]','',$now,'') \\g \n";
         
         push @output, ( $new );
      } # done with this zipcode block
   } # done reading through the "normal" portion of the table

   return @output;

} # UPS_zones done with readthrough of file

### parse_UPS_domestic_rates
##     takes an array of lines (file)
##     outputs an array of lines (for dump to file)
#
sub parse_UPS_dom_rates {
   ## Globals used - $shipmeth for shipping sku
   my( @lines,@output,@special,@temp,@zones );
   @lines=@_;
   @output=();
   @special=();
   
   ## format of .csv file data segment is:
   #  zip [-zip],Ground,3 Day Select,2nd Day Air,2nd Day Air AM,
   #     Next Day Air Saver, Next Day Air
   
   while (( $lines[0] !~ /Exceed/ ) and ( @lines )) {
      $l = shift @lines;
      print "$l \n";
   } # done chopping header lines

   if ( ! @lines ) { # Test for burn-through
       return ();
   } # done testing for file burn-through

   ## get zone list from this last header line
   #  note column 0 is not a zone
   $l = shift @lines;
   @zones = split /\,/, $l; 
   foreach $j ( 1..$#zones ) {
      $zones[$j] =~ tr/0123456789.//cd ;       # kill all non-numerics 
   } # done cleaning non-numeric zones

   ## run through remainder of file assuming all lines are same
   #
   while ( @lines ) {
      $l = shift @lines;
      # strip non-numeric non-delimiter characters
      # TBD - RF
      @temp = split /\,/, $l;
      $mass=$temp[0];         # first column is "not to exceed wt"
      next if ( $mass !~ /(\d|\.)+/ ) ;
      $mass.=".00" if ( $mass !~ /\d\./ );
      foreach $j ( 1..$#temp ) {
         $l = $temp[$j];
         $l =~ tr/0123456789.//cd ;       # kill all non-numerics 
         next if ( ! $l );
         next if ( $l =~ /\-/ );
         # construct the mysql insert statement
         $q = "INSERT INTO zonerate VALUES ";
         $q.= "('$shipmeth','$zones[$j]',$mass,$l,0.00,'','',$now,'') ";
         $q.= " \\g \n";
         push @output, ( $q );
      } # done running through zones      
      

   } # done with read-through
   

   ## TBD - add code here to read through the special cases section
   #        at the end of the file

   return @output;

} # done with parse_UPS_dom_rates

### parse_FedEd_domestic_rates
##     takes an array of lines (file)
##     outputs an array of lines (for dump to file)
#
sub parse_FedEx_dom_rates {
   ## Globals used - $shipmeth for shipping sku
   my( @lines,@output,@special,@temp,@zones );
   @lines=@_;
   @output=();
   @special=();
   
   ## format of .csv file data segment is:
   #  zip [-zip],Ground,3 Day Select,2nd Day Air,2nd Day Air AM,
   #     Next Day Air Saver, Next Day Air
   
   while (( $lines[0] !~ /lb/ ) and ( @lines )) {
      $l = shift @lines;
      print "$l \n";
   } # done chopping header lines

   if ( ! @lines ) { # Test for burn-through
       return ();
   } # done testing for file burn-through

   ## run through remainder of file assuming all lines are same
   #
   $nothing="";

   while ( @lines ) {
      $l = shift @lines;
      $l=~ s/lb./$nothing/g;
      $l=~ s/lbs./$nothing/g;
      # strip non-numeric non-delimiter characters
      # TBD - RF
      @temp = split /\ +/, $l;
      $mass=$temp[0];         # first column is "not to exceed wt"
      next if ( $mass !~ /(\d|\.)+/ ) ;
      $mass.=".00" if ( $mass !~ /\d\./ );
      if ($mass =~ /-/) {
	  ($mass1, $mass2) = split /\-/, $mass;
	  $mass=$mass2;
	  $mass2=~ s/+/$nothing/g;
      }
      
      foreach $j ( 1..$#temp ) {
         $l = $temp[$j];
         $l =~ tr/0123456789.//cd ;       # kill all non-numerics 
         next if ( ! $l );
         next if ( $l =~ /\-/ );
	
	 next if ($j==9 || $j > 13);
	 $zone=$j+1;
         # construct the mysql insert statement
         $q = "INSERT INTO zonerate VALUES ";
         $q.= "('$shipmeth','$zone',$mass,$l,0.00,'','',$now,'') ";
         $q.= " \\g \n";
         push @output, ( $q );
	 if ($zone == 9) {
	     $q = "INSERT INTO zonerate VALUES ";
	     $q.= "('$shipmeth','10',$mass,$l,0.00,'','',$now,'') ";
	     $q.= " \\g \n";
	     push @output, ( $q );
	 }
	 if ($zone == 13) {
	      $q = "INSERT INTO zonerate VALUES ";
	     $q.= "('$shipmeth','14',$mass,$l,0.00,'','',$now,'') ";
	     $q.= " \\g \n";
	      push @output, ( $q );

	      $q = "INSERT INTO zonerate VALUES ";
	     $q.= "('$shipmeth','15',$mass,$l,0.00,'','',$now,'') ";
	     $q.= " \\g \n";
	      push @output, ( $q );

	       $q = "INSERT INTO zonerate VALUES ";
	     $q.= "('$shipmeth','16',$mass,$l,0.00,'','',$now,'') ";
	     $q.= " \\g \n";
	      push @output, ( $q );
	      
	 }     
	     

      } # done running through zones      
      

   } # done with read-through
   
   return @output;

} # done with parse_UPS_dom_rates


1;
